/*************************************************************/
/* File        : cmdline.c                                   */
/* Author      : Paul Lemmens (paul.lemmens@student.kun.nl)  */
/* Description : Defines the keyword-value struct            */
/* License     : GNU GPL                                     */
/*************************************************************/

/*
wmseti is a Windowmaker dockapp that displays seti@home client statistics.
Copyright (C) 2000 Paul Lemmens

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <unistd.h>

#include "cmdline.h"
#include "wmsetirc-file.h"
#include "global.h"

/************************************************************************************/
/* parse_cmdline() parses the commandline of wmseti and puts the results in the
 * global variables from global.h. Returns 0 upon success and -1 upon failure and
 * 1 if help is requested. */
/************************************************************************************/
int parse_cmdline(int argc, char **argv, wmsetirc_t *rc)
{
	char *dir = NULL, *d, *usedir, *debuglvl = NULL;
	int c = 0;
	
	if ( DebugLevel >= 2 )
		fprintf(stderr, "\t\tParsing commandline %s\n", argv);

	/* Parse the commandline arguments. */
	opterr = 0;
	optind = 0;

	/* Options we recognize: <-D> <-V> <-v> <-h> <-d string> <-a>. */
	while ( (c = getopt(argc, argv, "D::Vvhad:")) != -1 )
	{
		switch ( c )
		{
			case 'v':
				Version = 1;
				break;
			case 'D':
				DebugLevel = 1;
				if ( optarg )
					debuglvl = (char *)strdup(optarg);
				break;
			case 'V':
				Verbose = 1;
				break;
			case 'h':
				Help = 1;
				break;
			case 'd':
				/* dir can contain multiple directories, separated by comma's. */
				dir = (char *)strdup(optarg);
				break;
			case 'a':
				Afterstep = 1;
				break;
			case '?':
				fprintf(stderr, "unrecognized option -%c\n", optopt);
				Help = 1;
				break;
		}
	}

	/* Tell if we found arguments that weren't of any use to use. */
	/* This doesn't seem necessary because of the "case '?'" in the
	 * previous switch. */
	for ( ; optind < argc; optind++ )
		fprintf(stderr, "ignoring argument: %s\n", argv[optind]);

	/* Check for arguments to the commandline arguments. DebugLevel can contain
	 * one (optional), if not present DebugLevel == 1;  directory must have
	 * one or more directories. */
	if ( debuglvl )
	{
		if ( DebugLevel >= 2 )
			fprintf(stderr, "\t\tFound debuglevelstring %s\n", debuglvl);

		/* I'd like to use strtod() here, but for some reason this function
		 * flunks. Therefore (for the time being use atoi() below.
		if ( (DebugLevel = strtod(debuglvl, (char *)NULL)) == 0 )
		{
			sprintf(ErrMsg, "converting string %s to an integer in parse_cmdline()", debuglvl);
			perror(ErrMsg);
			free(debuglvl);
			return -1;
		}
		*/
		DebugLevel = atoi(debuglvl);
		free(debuglvl);
		if ( DebugLevel < 0 || DebugLevel > 3 )
		{
			fprintf(stderr, "invalid debuglevel %d detected in parse_cmdline()\n", DebugLevel);
			return -1;
		}
	}

	/* Now finally check if the commandline contained a directory. Check the
	 * directory and put in the wmsetirc_t struct. Multiple directories are
	 * separated by comma's. */
	if ( dir )
	{
		if ( DebugLevel >= 2 )
			fprintf(stderr, "\t\tUsing directory argument %s.\n", dir);

		/* First make a backup of the directory arg, because strtok() destroys
		 * it argument. (Re)set numdirs to 0 (sneakily) to force overwrite of
		 * directories read in from configuration file. */
		d = (char *)strdup(dir);
		rc->numdirs = 0;
		usedir = (char *)strtok(d, ",");
		while ( usedir != NULL )
		{
			if ( insert_setidir(usedir, rc) != 0 )
			{
				fprintf(stderr, "inserting %s into setidirs[] in parse_cmdline()\n", usedir);
				free(dir);
				free(d);
				if (usedir)
					free(usedir);
				return -1;
			}
			usedir = (char *)strtok(NULL, ",");
		}
		free(dir);
		free(d);
	}

	return 0;
}

/************************************************************************************/
/* usage() displays a short summary of all commandline options and how to use these.
 * The parameter argv is needed to extract the programname. */
/************************************************************************************/
void usage(char **argv)
{
	char *programname, *p;

	/* Find the programname by stripping of the path. This way the function is
	 * very generic. Not really needed though. */
	p = (char *)strrchr(argv[0], '/');
	if ( p == NULL )
		programname = argv[0];
	else
		programname = (char *)(p+1);

	/* Now print how wmseti can be used. The name wmseti is contained in the
	 * variable programname. */
	fprintf(stderr, "usage: %s [-D [lvl]] [-V] [-v] [-h] [-a] [-d dir]\n", programname);
	fprintf(stderr, "where:\n");
	fprintf(stderr, "\t-D [lvl]\t: prints debug messages\n");
	fprintf(stderr, "\t-V\t\t: prints a verbose message\n");
	fprintf(stderr, "\t-v\t\t: prints the version number\n");
	fprintf(stderr, "\t-h\t\t: prints this information\n");
	fprintf(stderr, "\t-a\t\t: uses the Afterstep pixmap\n");
	fprintf(stderr, "\t-d dir\t\t: specify the SETI@home client dir\n");
	fprintf(stderr, "\nNote:\n");
	fprintf(stderr, "If -d is not specified, dir is taken from the wmsetirc file.\n");
	fprintf(stderr, "Commandline arguments override the wmsetirc configuration.\n");
	fprintf(stderr, "Debuglevel lvl ranges 0..3, with 0 no debugging and 3 much output.\n");

	return;
}
