/*************************************************************/
/* File        : dockapp.c                                   */
/* Author      : Paul Lemmens (paul.lemmens@student.kun.nl)  */
/*             : Dan Scott (dan.scott@acm.org)               */
/* Description : Defines the keyword-value struct            */
/* License     : GNU GPL                                     */
/*************************************************************/

/*
wmseti is a Windowmaker dockapp that displays seti@home client statistics.
Copyright (C) 2001 Paul Lemmens, Dan Scott

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

/* These includes need to be in this sequence, else strange errors. */
#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "dockapp.h"
#include "wmgeneral.h"
#include "statistics.h"
#include "pixmap.h"
#include "global.h"

/************************************************************************************/
/* blitstring() prints parameter what on the dockapp icon on position (x,y). x and y
 * are expressed in pixels. */
/************************************************************************************/
void blitstring(const char *what, const int x, const int y)
{
	int i = 0, c = 0, k = 0;

	/* Copy each individual char to the dockapp by copying it from the 
	 * pixmap. k is the x-coordinate of each individual char. */
	k = x;
	for ( i = 0; what[i]; i++ )
	{
		c = toupper(what[i]);
		
		/* [A-Z] is in sequence; the ascii value can be used to determine
		 * where the char is located in the pixmap. For '.', '/' and '%'
		 * this doesn't apply. */
		if ( c >= 'A' && c <= 'Z' )
		{
			c -= 'A';
			copyXPMArea(c*6,74, CHARWIDTH,CHARHEIGHT, k,y);
		}
		else if ( c == '.' )
			copyXPMArea(66,64, CHARWIDTH,CHARHEIGHT, k,y);
		else if ( c == '/' )
			copyXPMArea(72,64, CHARWIDTH,CHARHEIGHT, k,y);
		else if ( c == '%' )
			copyXPMArea(78,64, CHARWIDTH,CHARHEIGHT, k,y);
		else
		{
			c -= '0';
			copyXPMArea(c*6,64, CHARWIDTH,CHARHEIGHT, k,y);
		}
		k += 6;

		/* Setting the level to 4 effectively disables this debuggin info! */
		if ( DebugLevel >= 4 )
			fprintf(stderr, "\t\t\t\tBlitting char %c to (%d,%d)\n", c, k,y);
	}

	return;
}

/************************************************************************************/
/* blitnum() prints parameter what on the dockapp icon on position (x,y). x and y 
 * are expressed in pixels. */
/************************************************************************************/
void blitnum(const int what, const int x, const int y)
{
	char buffer[7];

	/* Just convert the integer to a string and then use blitstring() */
	if ( what > 99999 )
		return;
	sprintf(buffer, "%02i", what);

	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tConverted number %d to string %s.\n", what, buffer);

	blitstring(buffer, x, y);
	
	return;
}

/************************************************************************************/
/* right_align() determines the x-coord for starting point of a right aligned 
 * string in the dockapp. Returns the coordinate on success or -1 on failure.
 * The  right_align_int() does this for integers. */
/************************************************************************************/
int right_align(const char *what, const pixmap_t *pm)
{
	int len = 0, x = pm->lc;

	/* Determine the length of what and subtract as many chars from the final
	 * position to determine the starting position. */
	len = strlen(what);

	/* Zero length strings can't be aligned. Larger than 8 chars doesn't fit. */
	if ( len == 0 || len > 8 )
		return -1;

	x -= (len - 1) * CHARWIDTH;
	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tRight aligned starting position: %d\n", x);

	return x;
}

int right_align_int(const int what, const pixmap_t *pm)
{
	char buffer[7];

	/* Convert the integer to a string and use the other function. */
	if ( what > 99999 )
		return -1;
	sprintf(buffer, "%d", what);

	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tConverted number %d to string %s.\n", what, buffer);

	return (right_align(buffer, pm));
}

/************************************************************************************/
/* clearline() is a simple routine to clear a certain line of the dockapp.          */
/************************************************************************************/
void clearline(const int x, const int y, const pixmap_t *pm)
{
	/* First clear partially from the left, then from the right. */
	copyXPMArea(34+pm->xo,84, 5*CHARWIDTH,CHARHEIGHT, x,y);
	copyXPMArea(34+pm->xo,84, 5*CHARWIDTH,CHARHEIGHT, x+3*CHARWIDTH,y);
	copyXPMArea(34+pm->xo,84, 3*CHARWIDTH,CHARHEIGHT, pm->lc-3*CHARWIDTH,y);
	copyXPMArea(34+pm->xo,84, 1*CHARWIDTH,CHARHEIGHT, pm->lc,y);

	return;
}

/************************************************************************************/
/* print_pidid() prints a PID identifier depending on the length of the pid given.  */
/************************************************************************************/
void print_pidid(const pid_t pid, const int x, const int y)
{
	/* The current line is already cleared, so we should only have to display 
	 * the identifier. */
	if ( pid < 9999 )
		blitstring("PID:", x, y);
	else if ( pid < 99999 )
		blitstring("P:", x, y);

	return;
}

/************************************************************************************/
/* print_stats() prints the stats from rc->sequence into the dockapp. Returns 0
 * on success and -1 on failure. */
/************************************************************************************/
int print_stats(const stats_t *stats, wmsetirc_t *rc, const pixmap_t *pm)
{
	int i = 0, xcoord = pm->xo, ycoord = pm->yo;
	char buffer[25];
	
	/* For each line in the dockapp, extract the statistic from rc and the
	 * value from stats.*/
	for ( i = 0; i < ICON_LINES; i++ )
	{
		xcoord = pm->xo;
		ycoord = (i * LINEHEIGHT) + pm->yo;

		/* Clear the current line. */
		clearline(xcoord, ycoord, pm);

		/* Now fetch the statistic for this line and then do stuff with its value. */
		if ( fetch_sequence(i, rc, buffer) == NULL )
		{
			fprintf(stderr, "fetching sequence[%d] in print_stats()\n", i);
			return -1;
		}

		if ( strcmp(buffer, "title") == 0 )
		{
			/* If flashpid exceeds FLASH_SPAN change the title to the PID line. */
			if ( rc->flashpid > FLASH_SPAN )
			{
				print_pidid(stats->pid, xcoord, ycoord);
				xcoord = right_align_int(stats->pid, pm);
				sprintf(buffer, "%d", stats->pid);

				/* If another set of FLASH_SPAN cycles has gone by, switch
				 * back to the title. */
				if ( rc->flashpid > 2*FLASH_SPAN )
					rc->flashpid = 1;
			}
			else
				strcpy(buffer, (char *)"wmseti");

			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", buffer, xcoord, ycoord);
		}
		else if ( strcmp(buffer, "pid") == 0 )
		{
			print_pidid(stats->pid, xcoord, ycoord);
			xcoord = right_align_int(stats->pid, pm);
			sprintf(buffer, "%d", stats->pid);

			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", buffer, xcoord, ycoord);
		}
		else if ( strcmp(buffer, "prog") == 0 )
		{
			/* First make a single char from status, right align it and
			 * print it out. Print out progress here. */
			blitstring(stats->progress, xcoord,ycoord);

			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", stats->progress, xcoord, ycoord);

			if ( strcpy(buffer, stats->status) == NULL )
			{
				perror("copying status to buffer in print_stats()");
				return -1;
			}
			buffer[1] = '\0';
			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tCropped buffer to %s.\n", buffer);
			xcoord = right_align(buffer, pm);

			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", buffer, xcoord, ycoord);
		}
		else if ( strcmp(buffer, "cpu") == 0 )
		{
			strcpy(buffer, stats->cputime);
			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", buffer, xcoord, ycoord);
		}
		else if ( strcmp(buffer, "timeleft") == 0 )
		{
			strcpy(buffer, stats->timeleft);
			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", buffer, xcoord, ycoord);
		}
		else if ( strcmp(buffer, "nwus") == 0 )
		{
			blitstring("Rx:", xcoord, ycoord);
			xcoord = right_align_int(stats->nwus, pm);
			sprintf(buffer, "%d", stats->nwus);

			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", buffer, xcoord, ycoord);
		}
		else if ( strcmp(buffer, "nresults") == 0 )
		{
			blitstring("Tx:", xcoord, ycoord);
			xcoord = right_align_int(stats->nresults, pm);
			sprintf(buffer, "%d", stats->nresults);

			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tPutting %s on (%d,%d)\n", buffer, xcoord, ycoord);
		}
		else
		{
			sprintf(ErrMsg, "fetching data for %s in print_stats()", buffer);
			perror(ErrMsg);
			return -1;
		}
		
		/* Print the string to the icon. */
		blitstring(buffer, xcoord, ycoord);
	}

	return 0;
}

/************************************************************************************/
/* dockapp() is the function doing the dockapp routine. Returns 0 upon success or
 * -1 upon failure. */
/************************************************************************************/
int dockapp(wmsetirc_t *rc, const pixmap_t *pm, int argc, char **argv)
{
	XEvent Event;
	stats_t stats;

	/* Flags indicating what to do upon a certain mousepress. */
	int scrollsetidir = 0;
	int displaypid = 0;
	int stopseti = 0;
	
	/* Initialize the dockapp icon window. */
	if ( DebugLevel >= 1 )
		fprintf(stderr, "\tInitializing dockapp icon\n");
	createXBMfromXPM(mask_bits, pm->pixmap, MASK_WIDTH, MASK_HEIGHT);
	openXwindow(argc, argv, pm->pixmap, mask_bits, MASK_WIDTH, MASK_HEIGHT, pm->size, pm->border);
	RedrawWindow();

	/* Forever read the stats and display them. After displaying, handle the
	 * X events. */
	while ( 1 )
	{
		/* First read the statistics, the see if we need to carry out a mouse event
		 * from the previous cycle of the while loop. */
		read_stats(&stats, rc->currentdir);
		if ( stats.pid == -1 )
		{
			fprintf(stderr, "reading statistics from %s in dockapp()\n", rc->currentdir);
			return -1;
		}
		if ( DebugLevel >= 2 )
			fprintf(stderr, "\t\tRead statistics from dir %s\n", rc->currentdir);

		/* See if on of the mouse events needs to be carried out. */
		if ( stopseti )
		{
			/* First see in what context we are. If the client is already stopped
			 * restart it, else stop it. Then carry it out the action. */
			if ( stats.status[0] == 'R' )
				rc->signal = SIGSTOP;
			else
				rc->signal = SIGCONT;
			
			if ( DebugLevel >= 2 )
				fprintf(stderr, "\t\tKilling PID %d with signal %d\n", stats.pid, rc->signal);

			kill(stats.pid, rc->signal);

			if ( DebugLevel >= 3 )
				fprintf(stderr, "\t\t\tCurrent signal %d\n", rc->signal);

			stopseti = 0;
		}

		if ( displaypid )
		{
			/* Do a direct displaying of the pid without the use
			 * of print_stats(). Don't forget to redraw else the
			 * changes won't take effect. */
			/* For the moment the pid is always displayed on line 0
			 * of the dockapp. */
			if ( !rc->flashpid )
			{
				clearline(pm->xo,pm->yo, pm);
				print_pidid(stats.pid, pm->xo,pm->yo);
				blitnum(stats.pid, right_align_int(stats.pid, pm),pm->yo);

				if ( DebugLevel >= 2 )
					fprintf(stderr, "\t\tPrinting %d on (%d,%d)\n", stats.pid, right_align_int(stats.pid, pm),pm->yo);
			}
			else
			{
				clearline(pm->xo,pm->yo+LINEHEIGHT, pm);
				blitstring(VERSION, pm->xo, pm->yo+LINEHEIGHT);

				if ( DebugLevel >= 2 )
					fprintf(stderr, "\t\tPrinting %d on (%d,%d)\n", VERSION, pm->xo, pm->yo+LINEHEIGHT);
			}
			RedrawWindow();
			usleep(DISPLAYPID_SPAN);
			displaypid = 0;
		}

		if ( scrollsetidir )
		{
			if ( scrollsetidirs(rc) != 0 )
			{
				fprintf(stderr, "scrolling to the next setidir in dockapp()\n");
				return -1;
			}
			if ( DebugLevel >= 2 )
				fprintf(stderr, "\t\tWent to dir %s\n\tfrom dir %s\n", rc->currentdir, rc->setidirs[rc->currentdirnum-1]);

			scrollsetidir = 0;
		}

		/* Now print these statistics in the dockapp icon and redraw the window
		 * to make the changes effective. */
		if ( print_stats(&stats, rc, pm) != 0 )
		{
			fprintf(stderr, "printing the stats to the dockapp icon in dockapp()\n");
			return -1;
		}
		RedrawWindow();
		if ( DebugLevel >= 1 )
			fprintf(stderr, "\tPrinted statistics on dockapp\n");

		/* Handle the X Events. Events are the default ones plus the 
		 * mousepresses. Left: scroll through setidirs. Middle: display
		 * PID of current seticlient. Right: stop wmseti. */
		while ( XPending(display) )
		{
			XNextEvent(display, &Event);
			switch ( Event.type )
			{
				case Expose:
					RedrawWindow();
					break;
				case DestroyNotify:
					XCloseDisplay(display);
					exit(0);
					break;
				case ButtonRelease:
					/* From left to right the buttons are numbered
					 * (resp.): 1, 2, 3. */
					if ( Event.xbutton.button == 1 && rc->numdirs > 1 )
						scrollsetidir = 1;
					if ( Event.xbutton.button == 2 )
						displaypid = 1;
					if ( Event.xbutton.button == 3 )
						stopseti = 1;
					break;
			}
			usleep(100000L);
		}
		usleep(100000L);
		if ( DebugLevel >= 1 )
			fprintf(stderr, "\tHandled X events\n");

		/* Update the cycle counter for flashpid functionality. */
		if ( rc->flashpid > 0 )
			rc->flashpid++;
	}

	return 0;
}
