/*************************************************************/
/* File        : files-dirs.c                                */
/* Author      : Paul Lemmens (paul.lemmens@student.kun.nl)  */
/* Description : Defines the keyword-value struct            */
/* License     : GNU GPL                                     */
/*************************************************************/

/*
wmseti is a Windowmaker dockapp that displays seti@home client statistics.
Copyright (C) 2000 Paul Lemmens

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <sys/types.h>

#include "global.h"
#include "files-dirs.h"

/************************************************************************************/
/* expand_dir tries to expand its argument to an absolute directory. Can be used to
 * create a full path out of $HOME/foobar. Returns the directory upon success
 * or NULL upon failure. */
/************************************************************************************/
char *expand_dir(char *dir)
{
	char *home, remaining_dir[HOME_LEN], buffer[HOME_LEN];
	struct passwd pswd;
	struct passwd *pw = &pswd;
	int i = 0, firstslash = 0;

	if ( DebugLevel >= 2 )
		fprintf(stderr, "\t\tTrying to expand %s\n", dir);

	/* expand_dir handles thingies starting with dollar-HOME, tilde-user, 
	 * tilde-slash, and slash. There shouldn't be any fancy schmancy behind 
	 * these four. If the fourth is found asume an absolute path. In the other 
	 * cases, first try the environment and then the passwd file. */

	/* If char 0 is slash the directory is already absolute and
	 * nothing needs to be done. */
	if ( *dir == '/' )
		return dir;

	/* Allocate memory for home, so $HOME or ~user can be inserted
	 * into them. 128 should be plenty because only the homedir is inserted. */
	if ( (home = malloc(HOME_LEN*sizeof(char))) == NULL )
	{
		perror("allocating memory for home in expand_dir()");
		return NULL;
	}

	/* First set some default information. */
	/* Locate first slash and copy on from there. */
	while ( *(dir+i) != '/' )
		i++;
	firstslash = i;
	if ( strcpy(remaining_dir, dir+i) == NULL )
	{
		perror("copying remaining dir in expand_dir()");
		return NULL;
	}
	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tremaining_dir: %s\n", remaining_dir);

	/* Fetch our own $HOME. */
	if ( strcpy(home, getenv("HOME")) == NULL )
	{
		/* On this system $HOME is not set. Read passwd-file. */
		if ( (pw = getpwuid(getuid())) != NULL )
		{
			if ( (home = realloc(home, (strlen(pw->pw_dir)+1)*sizeof(char))) == NULL )
			{
				sprintf(ErrMsg, "reallocing home from %d to %d in expand_dir()", strlen(home), strlen(pw->pw_dir)+1);
				perror(ErrMsg);
				free(home);
				return NULL;
			}
			if ( strcpy(home, pw->pw_dir) ==  NULL )
			{
				sprintf(ErrMsg, "copying %s to home in expand_dir()", pw->pw_dir);
				perror(ErrMsg);
				free(home);
				return NULL;
			}
			if ( DebugLevel >= 2 )
				fprintf(stderr, "\t\tUsing %s for home\n", home);
		}
		else
		{
			sprintf(ErrMsg, "failed to getpwuid in expand_dir()\nunable to determine home dir in expand_dir()\n");
			perror(ErrMsg);
			free(home);
			return NULL;
		}
	}

	/* See if somebody elses home is given. */
	if ( *dir == '~' && *(dir+1) != '/' )
	{
		i = 1;
		while ( *(dir+i) != '/' )
		{
			buffer[i-1] = *(dir+i);
			i++;
		}
		buffer[i-1] = '\0';
		if ( DebugLevel >= 3 )
			fprintf(stderr, "\t\t\tTrying to determine ~%s\n", buffer);
		if ( (pw = getpwnam(buffer)) == NULL )
		{
			sprintf(ErrMsg, "failed to getpwnam for %s in expand_dir()\nunable to determine homedir in expand_dir()", buffer);
			perror(ErrMsg);
			free(home);
			return NULL;
		}
		
		/* Now overwrite original home, because tilde-user can be different from
		 * $HOME. */
		if ( (home = realloc(home, (strlen(pw->pw_dir)+1)*sizeof(char))) == NULL )
		{
			sprintf(ErrMsg, "reallocating mem for home from %d to %d in expand_dir()", strlen(home), strlen(pw->pw_dir)+1);
			perror(ErrMsg);
			free(home);
			return NULL;
		}
		if ( strcpy(home, pw->pw_dir) == NULL )
		{
			sprintf(ErrMsg, "copying %s to home in expand_dir()", pw->pw_dir);
			perror(ErrMsg);
			free(home);
			return NULL;
		}
		if ( DebugLevel >= 2 ) 
			fprintf(stderr, "\t\tUsing home %s\n", home);
	}

	/* home now contains a perfect equivalent of either dollar-HOME, tilde-slash, or tilde-user. So
	 * remaining_dir and home can be simply appended. */
	if ( (dir = realloc(dir, (strlen(home)+strlen(remaining_dir)+1)*sizeof(char))) == NULL )
	{
		sprintf(ErrMsg, "reallocating dir from %d to %d in expand_dir()", strlen(dir), strlen(home)+strlen(remaining_dir)+1);
		perror(ErrMsg);
		free(home);
		return NULL;
	}
	sprintf(dir, "%s%s", home, remaining_dir);
	if (DebugLevel >= 2 )
		fprintf(stderr, "\t\treturned dir %s in expand_dir()\n", dir);

	free(home);
	return dir;
}

/************************************************************************************/
/* existing_dir tries to verify if parameter dir is existant on the current system.
 * Returns 0 upon success, and anything else upon failure. */
/************************************************************************************/
int existing_dir(const char *dir)
{
	DIR *dp;
	FILE *fp;

	/* dir can be something like /etc/ but also something like /etc/wmsetirc.
	 * First test former possibility and then the latter one. */
	if ( DebugLevel >= 1 )
		fprintf(stderr, "\tTrying to find %s\n", dir);
	if ( (dp = opendir(dir)) == NULL )
	{
		/* Filename appended? */
		if ( (fp = fopen(dir, "r")) == NULL )
		{
			/* Not even a file appended. Must be nonsense. */
			sprintf(ErrMsg, "could not locate file %s in existing_dir()", dir);
			perror(ErrMsg);
			closedir(dp);
			return 1;
		}
		fclose(fp);
	}
	closedir(dp);

	return 0;
}
