/*************************************************************/
/* File        : key-val.c                                   */
/* Author      : Paul Lemmens (paul.lemmens@student.kun.nl)  */
/* Description : Defines the keyword-value struct            */
/* License     : GNU GPL                                     */
/*************************************************************/

/*
wmseti is a Windowmaker dockapp that displays seti@home client statistics.
Copyright (C) 2000 Paul Lemmens

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "key-val.h"
#include "global.h"

/************************************************************************************/
/* split_line() splits line around splitchar. The string on the left of splitchar is
 * put into the keyword member of a keyval_t struct. The string on the right of the
 * splitchar is put into the value member. A keyval_t struct is returned. On error
 * both members of the struct are set to "". */
/************************************************************************************/
keyval_t split_line(const char *splitchar, const char *line)
/* consider using strtok for this function (but be careful because it */
/* destroys its buffer) - agw */
{
	keyval_t kv_pair;
	int i = 0;
	char *c;

	/* Set a default value for kv_pair to use on error. */
	kv_pair.keyword[0] = '\0';
	kv_pair.value[0] = '\0';

	/* Deuglify (strip \n) line specially for debug line. */
	/* Watch out, this strchr() thingie is kinda sneaky, 'cause it writes, as a
	 * sort of side effect, to a const char *. Nasty way to do this! */
	if ( DebugLevel >= 3 )
	{
		c = strchr(line, '\n');
		if ( c )
			*c = '\0';
		fprintf(stderr, "\t\t\tSplitting `%s' around `%s'\n", line, splitchar);
	}

	/* Char-copy until we find splitchar. If we're past the length of 
	 * line, the line isn't correctly formatted. */
	while ( (strncmp(line+i, splitchar, 1) != 0) && (i < strlen(line)) )
	{
		strncat(kv_pair.keyword, line+i, 1);
		i++;
	}

	/* Check on what condition the while loop stopped. i can be out of 
	 * bounds or zero (resp. no value or no keyword). */
	if ( i > strlen(line) )
	{
		/* Watch out, see the above remark on this code. */
		c = strchr(line, '\n');
		if ( c )
			*c = '\0';
		fprintf(stderr, "no `=' found in line %s in split_line()\n", line);
		kv_pair.keyword[0] = '\0';
		kv_pair.value[0] = '\0';
		return kv_pair;
	}
	else if ( i == 0 )
	{
		c = strchr(line, '\n');
		if ( c )
			*c = '\0';
		fprintf(stderr, "no keyword found in line %s in split_line()\n", line);
		kv_pair.keyword[0] = '\0';
		kv_pair.value[0] = '\0';
		return kv_pair;
	}

	/* So from here on we have found a keyword. Now skip the splitchar and
	 * terminate the keyword in the struct. */
	kv_pair.keyword[i++] = '\0';

	/* Now copy remaining part into value member, but chop off terminating '\n'. */
	if ( strcpy(kv_pair.value, line+i) == NULL )
	{
		sprintf(ErrMsg, "copying value of line %s in split_line()", line);
		perror(ErrMsg);
		kv_pair.keyword[0] = '\0';
		kv_pair.value[0] = '\0';
		return kv_pair;
	}
	c = strchr(kv_pair.value, '\n');
	if ( c )
		*c = '\0';

	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tSplit results: %s ==> %s\n", kv_pair.keyword, kv_pair.value);

	return kv_pair;
}

/************************************************************************************/
/* is_keyword() tries to locate the first argument kw in the second argument kwlist.
 * If it is found, kw is a valid keyword and 0 is returned; if not 1 is returned.
 * This is also returned for errors. */
/************************************************************************************/
int is_keyword(const char *kw, const char *kwlist)
{
	if ( strstr(kwlist, kw) == NULL )
		return 1;
	else
		return 0;
}
