/*************************************************************/
/* File        : osdep.c                                     */
/* Author      : Paul Lemmens (paul.lemmens@student.kun.nl)  */
/* Description : Implements osdep functions from osdep.h     */
/* License     : GNU GPL                                     */
/*************************************************************/

/*
wmseti is a Windowmaker dockapp that displays seti@home client statistics.
Copyright (C) 2000 Paul Lemmens

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#ifdef __sun__
#include <procfs.h>
#endif

#include "osdep.h"
#include "global.h"

/************************************************************************************/
/* get_status() tries to determine the status of the process identified by pid. If
 * found, Returns a string starting with a single char describing the state, a space
 * and a description between () thingies. Upon failure NULL is returned. */
/************************************************************************************/
char *get_status(const pid_t pid, char *dest)
{
#ifdef __FreeBSD__
	FILE *fp;
	char command[50], buffer[128];
	int i = 0;
	char *statusptr;
#endif
#ifdef	__sun__
	FILE *fp;
	char buffer[128];
	psinfo_t ps;
#endif 
#ifdef __linux__
	FILE *fp;
	char buffer[128];
	int i = 0;
#endif 

	/* Set stat to the unknown state so we can bail out safely. */
	if ( strcpy(dest, (char *)"U (Unknown)") == NULL )
	{
		perror("initializing status in get_status()");
		return NULL;
	}

	/* We do this separately for each OS wmseti knows. Currently these are 
	 * Linux, FreeBSD and Solaris. See below for unknown OSses. */

#ifdef __FreeBSD__

	/* FreeBSD is a bit more difficult in retrieving the process state.
	 * Use the non-advised method from the Unix Programmers FAQ (1.8) of
	 * popen("ps -p %d", "r"). */

	/* If the PID is zero, the client isn't running, so don't get the
	 * process state. */
	if ( pid == 0 )
		return dest;

	/* Create and open the command. */
	sprintf(command, "ps -lp %d 2> /dev/null", pid);
	if ( (fp = popen(command, "r")) == NULL )
	{
		perror("erroring running ps command in get_status(), FreeBSD part");
		return dest;
	}

	/* Read the first line and try to find the status column (marked by STAT in
	 * FreeBSD. */
	if ( !fgets(buffer, sizeof(buffer), fp) )
	{
		perror("error reading from ps -lp output in get_status(), FreeBSD part");
		fprintf(stderr, "\tline was %s\n", buffer);
		fclose(fp);
		return dest;
	}
	if ( (statusptr = strstr(buffer, "STAT")) == NULL )
	{
		sprintf(ErrMsg, "retrieving STAT column from %s in get_status(), FreeBSD part", buffer);
		perror(ErrMsg);
		fclose(fp);
		return dest;
	}
	i = (statusptr - line) * sizeof(char);

	/* The next line will contain the status character. */
	if ( !fgets(buffer, sizeof(buffer), fp) )
	{
		perror("error reading 2nd line from ps -lp output in get_status(), FreeBSD part");
		if ( DebugLevel )
			fprintf(stderr, "\tline was %s\n", buffer);
		fclose(fp);
		return dest;
	}

	/* Copy the statuschar thingie. */
	while ( buffer[i] == ' ' )
		i++;
	sprintf(dest, "%c %s", line[i], "(FreeBSD)");
	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tReturned status (FreeBSD): %s\n", dest);

	fclose(fp);
#endif /* __FreeBSD__ */

#ifdef __sun__

	/* Sun uses a psinfo_t struct to read out the status of a process wiht
	 * PID from the file /proc/PID/psinfo. */

	/* If the PID is zero, no client is running. */
	if ( pid == 0 )
		return NULL;

	/* Open the psinfo file and try to read it. */
	sprintf(buffer, "/proc/%d/psinfo", pid);
	if ( (fp = fopen(buffer, "r")) != NULL )
	{
		if ( read(fileno(fp), &ps, sizeof(psinfo_t)) != -1 )
			sprintf(dest, "%c %s", ps.pr_lwp.pr_sname, "(SunOS)");
		else
		{
			sprintf(ErrMsg, "reading from /proc/%d/psinfo in get_status(), SunOS part", pid);
			perror(ErrMsg);
			fclose(fp);
			return dest;
		}
	}
	else
	{
		sprintf(ErrMsg, "opening /proc/%d/psinfo in get_status(), SunOS part", pid);
		perror(ErrMsg);
		return dest;	
	}
	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tReturned status (SunOS): %s\n", dest);

	fclose(fp);
#endif /* __sun__ */

#ifdef __linux__

	/* On Linux the status of a PID can be found on the second line of the
	 * file /proc/PID/status, in the form of "C (description)". */

	/* If the PID equals zero it's known the client isn't running. So die
	 * gracefully (!). */
	if ( pid == 0 )
		return NULL;

	/* On linux the file /proc/PID/status contains a line describing the state
	 * of process with PID on line 2 of the file. */
	sprintf(buffer, "/proc/%d/status", pid);
	if ( (fp = fopen(buffer, "r")) == NULL )
	{
		/* Actually we don't want any action here, especially returning NULL,
		 * because that disrupts the functioning of wmseti. So just a message. */
		if ( DebugLevel )
		{
			fprintf(stderr, "\tCouldn't open %s in get_status(), linux part\n", buffer);
			fprintf(stderr, "\tAssuming Unknown process status\n");
		}
		return dest;
	}

	for ( i = 0; i < 2; i++ )
	{
		if ( !fgets(buffer, sizeof(buffer), fp) )
		{
			sprintf(ErrMsg, "reading /proc/%d/status in get_status(), linux part", pid);
			perror(ErrMsg);
			fclose(fp);
			return dest;
		}
	}
	fclose(fp);

	/* The buffer now contains this line. Split it up and put the things
	 * in the respective members of status_t struct. */
	if ( !strncpy(dest, buffer+7, 15) )
	{
		sprintf(ErrMsg, "copying %s to dest in get_status(), linux part", buffer+7);
		perror(ErrMsg);
		return NULL;
	}
	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tReturned status (Linux): %s\n", dest);
#endif /* __linux__ */

	/* For unknown OSses we've already set the status to unknown. Just return this always. */
	return dest;
}
