/*************************************************************/
/* File        : statistics.c                                */
/* Author      : Paul Lemmens (paul.lemmens@student.kun.nl)  */
/*             : Dan Scott (dan.scott@acm.org)               */
/* Description : Implements statistics.h for reading stats   */
/* License     : GNU GPL                                     */
/*************************************************************/

/*
wmseti is a Windowmaker dockapp that displays seti@home client statistics.
Copyright (C) 2001 Paul Lemmens, Dan Scott

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/types.h>

#include "statistics.h"
#include "global.h"
#include "key-val.h"
#include "conversions.h"
#include "osdep.h"

/***********************************************************************************/
/* init_stats() initializes a stats_t struct stats to default values, so no strange
 * things happen in read_stats() isn't successful. Returns stats (stats_t struct)
 * initialized. If it fails, at least, the pid member should be -1. */
/***********************************************************************************/
stats_t *init_stats(stats_t *stats)
{
	if ( DebugLevel >= 1 )
		fprintf(stderr, "\tInitializing a statistics struct\n");

	/* Set the pid default to -1 to indicate an error. */
	stats->pid = -1;

	/* Set progress, cputime and timeleft to all zero values. */
	if ( strcpy(stats->progress, (char *)"00.00%") == NULL )
	{
		/* We found an error. The -1 in pid indicates this for catching. */
		perror("initializing stats->progress in init_stats()");
		return stats;
	}
	if ( strcpy(stats->cputime, (char *)"00:00:00") == NULL )
	{
		perror("initializing stats->cputime in init_stats()");
		return stats;
	}
	if ( strcpy(stats->timeleft, (char *)"00:00:00") == NULL )
	{
		perror("initializing stats->timeleft in init_stats()");
		return stats;
	}
	
	/* These can be simply set to zero. */
	stats->nwus = 0;
	stats->nresults = 0;

	/* Set the default status of the SETI@home client to Unknown. */
	if ( strcpy(stats->status, (char *)"U (Unknown)") == NULL )
	{
		perror("initializing stats->stats in init_stats()");
		return stats;
	}

	/* Now just before the return set pid to a real value. */
	stats->pid = 0;

	return stats;
}

/***********************************************************************************/
/* get_pid() extracts the PID of the current seti client and returns it. The current 
 * seti client is indicated by dir. Returns a valid pid upon success or -1 
 * upon failure. */
/***********************************************************************************/
pid_t get_pid(const char *dir)
{
	FILE *fp;
	pid_t pid = 0;
	char buffer[128];
	
	/* Create the PID file and open it. Its only line only contains the pid. */
	sprintf(buffer, "%s/%s", dir, PID_FILE);
	if ( DebugLevel >= 1 )
		fprintf(stderr, "\tUsing %s to find pid in get_pid()\n", buffer);
	if ( (fp = fopen(buffer, "r")) == NULL )
	{
		sprintf(ErrMsg, "error opening %s in get_pid()", buffer);
		perror(ErrMsg);
		return -1;
	}

	/* Read a single line and try to convert it to an integer. Don't check if
	 * the PID is valid and stuff. */
	if ( !fgets(buffer, sizeof(buffer), fp) )
	{
		sprintf(ErrMsg, "reading from %s in get_pid()", buffer);
		perror(ErrMsg);
		fclose(fp);
		return -1;
	}
	pid = atoi(buffer);
	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tConverted %s to integer %d\n", buffer, pid);
	if ( !pid ) 
	{
		sprintf(ErrMsg, "converting %s to a valid int in get_pid()", buffer);
		perror(ErrMsg);
		fclose(fp);
		return -1;
	}

	fclose(fp);
	return pid;
}

/***********************************************************************************/
/* read_workunit_stats() reads the STATE_FILE and tries to found the progress and
 * cputime. Sets the appropriate members to these values (a strings). Returns the
 * struct upon success and the struct with a pid of -1 upon failure. */
/***********************************************************************************/
stats_t *read_workunit_stats(const char *dir, stats_t *stats)
{
	FILE *fp;
	char buffer[MAX_VAL_LEN];
	keyval_t kv_pair;
	double tmp = 0, timedone = 0, progress = 0;

	/* Concatenate dir and file. Read until a line with either "prog" or
	 * "cpu" is found. */
	sprintf(buffer, "%s/%s", dir, STATE_FILE);
	if ( DebugLevel >= 1 )
		fprintf(stderr, "\tUsing %s to read the workunit stats.\n", buffer);
	if ( (fp = fopen(buffer, "r")) == NULL )
	{
		/* Because the new v2 seticlient removes this file if a unit is
		 * completed, we don't want to return a fault here. Downside:
		 * can't check anymore if something serious is going on. */
		if ( DebugLevel )
			fprintf(stderr, "\tCouldn't open %s. Workunit finished?\n", STATE_FILE);
		if ( strstr(stats->progress, "90.") == NULL )
		{
			sprintf(ErrMsg, "workunit not finished opening %s for reading in read_workunit_stats()", STATE_FILE);
			perror(ErrMsg);
		}
		return stats;
	}

	/* This while loop assumes the existance of the keywords in the file and
	 * does not check for a failure on finishing the loop. Dangerous. */
	while ( !feof(fp) )
	{
		/* No real error reporting here since it can disrupt the functioning
		 * of the dockapp icon. */
		if ( !fgets(buffer, sizeof(buffer), fp) )
			break;

		/* Check here if we've got a valid keyword or not. Once located, put the value
		 * in the stats_t struct. */
		kv_pair = split_line("=", buffer);
		if ( is_keyword(kv_pair.keyword, (char *)WORKUNIT_KEYWORDS) == 0 )
		{
			if ( strcmp(kv_pair.keyword, "cpu") == 0 )
			{
				/* stats.cputime[] needs to be a string. */
				if ( (tmp = strtod(kv_pair.value, (char **)NULL)) == 0 )
				{
					sprintf(ErrMsg, "converting %s to double in read_workunit_stats()", kv_pair.value);
					perror(ErrMsg);
					fclose(fp);
					stats->pid = -1;
					return stats;
				}

				/* Save the seconds for later use to calculate the timeleft member. */
				timedone = tmp;
				secs_2_time(tmp, stats->cputime);
				if ( DebugLevel >= 3 )
					fprintf(stderr, "\t\t\tConverted %f to %s\n", tmp, stats->cputime);
				if ( strcmp(stats->cputime, "99:99:99") == 0 )
				{
					sprintf(ErrMsg, "converting cputime %f to a time in read_workunit_stats()", tmp);
					perror(ErrMsg);
					fclose(fp);
					stats->pid = -1;
					return stats;
				}
			}
			else if ( strcmp(kv_pair.keyword, "prog") == 0 )
			{
				/* Same goes for progress. Needs to be a string. */
				if ( (tmp = strtod(kv_pair.value, (char **)NULL)) == 0 )
				{
					sprintf(ErrMsg, "converting %s to double in read_workunit_stats()", kv_pair.value);
					perror(ErrMsg);
					fclose(fp);
					stats->pid = -1;
					return stats;
				}

				/* First save the progress to calculate the timeleft later. */
				progress = tmp;
				tmp *= 100;
				sprintf(stats->progress, "%.2f%%", tmp);
				if ( DebugLevel >= 3 )
					fprintf(stderr, "\t\t\tConverted %f to %s\n", tmp, stats->progress);
			}
		}
	}
	fclose(fp);

	/* Now calculate the timeleft. First in seconds and then convert it to a string. */
	tmp = timedone/progress - timedone;
	secs_2_time(tmp, stats->timeleft);
	if ( DebugLevel >= 3 )
		fprintf(stderr, "\t\t\tConverted %f to %s\n", tmp, stats->timeleft);
	if ( strcmp(stats->timeleft, "99:99:99") == 0 )
	{
		sprintf(ErrMsg, "converting timeleft %f to a time in read_workunit_stats()", tmp);
		perror(ErrMsg);
		stats->pid = -1;
		/* No return needed, see below. */
	}

	return stats;
}

/***********************************************************************************/
/* read_user_stats() reads the userstats from USERINFO_FILE in dir. Puts the values
 * for nwus and nresults in stats and returns this. Upon failure the pid member
 * of stats is set to -1 and returned. */
/***********************************************************************************/
stats_t *read_user_stats(const char *dir, stats_t *stats)
{
	FILE *fp;
	char buffer[128];
	keyval_t kv_pair;

	/* concatenate the directory and file, open and read until nwus or nresults
	 * is found, or EOF is found. */
	sprintf(buffer, "%s/%s", dir, USERINFO_FILE);
	if ( DebugLevel >= 1 )
		fprintf(stderr, "\tUsing %s to read user stats\n", buffer);
	if ( (fp = fopen(buffer, "r")) == NULL )
	{
		/* Because the new v2 seticlient removes this file if a unit is
		 * completed, we don't want to return a fault here. Downside:
		 * can't check anymore if something serious is going on. */
		if ( DebugLevel >= 1 )
			fprintf(stderr, "\tCouldn't open %s. Workunit finished?\n", USERINFO_FILE);
		if ( strstr(stats->progress, "90.") == NULL )
		{
			sprintf(ErrMsg, "Workunit not finished opening %s for reading in read_user_stats()", USERINFO_FILE);
			perror(ErrMsg);
		}
		return stats;
	}

	/* This while loop assumes the existance of the keywords in the file and
	 * does not check for a failure on finishing the loop. Dangerous. */
	while ( !feof(fp) )
	{
		/* Do not return with an error value here, because it would
		 * disrupt the functioning of the dockapp. */
		if ( !fgets(buffer, sizeof(buffer), fp) )
			break;

		/* Now check if the keyword on the current line is valid, and if so
		 * store it in stats. */
		kv_pair = split_line("=", buffer);
		if ( is_keyword(kv_pair.keyword, (char *)USERINFO_KEYWORDS) == 0 )
		{
			if ( strcmp(kv_pair.keyword, "nwus") == 0 )
			{
				/* the value of nwus is a string but needs to be an
				 * integer. Convert. */
				stats->nwus = atoi(kv_pair.value);
				if ( DebugLevel >= 3 )
					fprintf(stderr, "\t\t\tConverted %s to %d\n", kv_pair.value, stats->nwus);
			}
			else if ( strcmp(kv_pair.keyword, "nresults") == 0 )
			{
				/* nresults also is a string needing to be
				 * converted to an integer. */
				stats->nresults = atoi(kv_pair.value);
				if ( DebugLevel >= 3 )
					fprintf(stderr, "\t\t\tConverted %s to %d\n", kv_pair.value, stats->nresults);
			}
		}
	}
	fclose(fp);

	return stats;
}

/***********************************************************************************/
/* read_stats() reads all statistics need to fill a stats_t struct. The function
 * calls the individual read* functions and puts the result together in stats. If
 * everything goes well stats is returned. If something fails the pid member is set
 * to -1. dir indicates the directory from which the data need to be gathered. */
/***********************************************************************************/
stats_t *read_stats(stats_t *stats, const char *dir)
{
	char status[32];

	if ( DebugLevel >= 2 )
		fprintf(stderr, "\t\tReading statistics from directory %s\n", dir);

	/* First initialize stats to set all values to their defaults. */
	init_stats(stats);
	if ( stats->pid == -1 )
	{
		fprintf(stderr, "initializing stats in read_stats()\n");
		return stats;
	}

	/* Now collect all values. Some functions return a single value,
	 * some put their results immediately in stats. */
	stats->pid = get_pid(dir);
	if ( stats->pid == -1 )
	{
		fprintf(stderr, "reading PID in read_stats()\n");
		return stats;
	}
	if ( get_status(stats->pid, status) == NULL )
	{
		fprintf(stderr, "getting status for PID %d in read_stats()\n", stats->pid);
		stats->pid = -1;
		return stats;
	}
	if ( strcpy(stats->status, status) == NULL )
	{
		sprintf(ErrMsg, "copying status %s to stats->status in read_stats()", status);
		perror(ErrMsg);
		stats->pid = -1;
		return stats;
	}
	read_workunit_stats(dir, stats);
	if ( stats->pid == -1 )
	{
		fprintf(stderr, "reading workunit stats in read_stats()\n");
		return stats;
	}
	read_user_stats(dir, stats);
	if ( stats->pid == -1 )
	{
		fprintf(stderr, "reading userinfo stats in read_stats()\n");
		return stats;
	}

	return stats;
}
